@@ -4,7 +4,7 @@ using System;
 using System.Collections;
 using System.Runtime.InteropServices;
 
-
+namespace FSpot {
 public class ImageView : Layout {
 	[DllImport ("libfspot")]
 	static extern IntPtr f_image_view_new ();
@@ -12,7 +12,7 @@ public class ImageView : Layout {
 	[DllImport ("libgobject-2.0-0.dll")]
 	static extern uint g_signal_connect_data (IntPtr obj, String name, SelectionChangedDelegate cb, int key, IntPtr p, int flags);
 
-	public ImageView () : base ()
+	public ImageView () : base (null, null)
 	{
 		Raw = f_image_view_new ();
 
@@ -169,3 +169,4 @@ public class ImageView : Layout {
 	public delegate void SelectionChangedHandler ();
 	public event SelectionChangedHandler SelectionChanged;
 }
+}
