@@ -103,6 +103,16 @@ public class Photo : DbItem, IComparable {
 		}
 	}
 
+	private bool loaded = false;
+	public bool Loaded {
+		get {
+			return loaded;
+		}
+		set {
+			loaded = value;
+		}
+	}
+
 	private string description;
 	public string Description {
 		get {
@@ -113,14 +123,12 @@ public class Photo : DbItem, IComparable {
 		}
 	}
 
-
 	// Version management
-
 	public const int OriginalVersionId = 1;
-
 	private uint highest_version_id;
 
 	private Hashtable version_names = new Hashtable ();
+
 	public uint [] VersionIds {
 		get {
 			uint [] ids = new uint [version_names.Count];
@@ -163,8 +171,7 @@ public class Photo : DbItem, IComparable {
 					       + " (" + version_name + ")" + extension);
 	}
 
-	public bool VersionNameExists (string version_name)
-	{
+	public bool VersionNameExists (string version_name)	{
 		foreach (string n in version_names.Values) {
 			if (n == version_name)
 				return true;
@@ -194,10 +201,15 @@ public class Photo : DbItem, IComparable {
 
 	public void DeleteVersion (uint version_id)
 	{
-		if (version_id == OriginalVersionId)
+		DeleteVersion (version_id, false);
+	}
+
+	public void DeleteVersion (uint version_id, bool remove_original)
+	{
+		if (version_id == OriginalVersionId && !remove_original)
 			throw new Exception ("Cannot delete original version");
 
-		string path = GetPathForVersionName (GetVersionName (version_id));
+		string path = GetVersionPath (version_id);
 		File.Delete (path);
 		PhotoStore.DeleteThumbnail (path);
 
@@ -319,7 +331,6 @@ public class Photo : DbItem, IComparable {
 
 
 	// Constructor
-
 	public Photo (uint id, uint unix_time, string directory_path, string name)
 		: base (id)
 	{
@@ -347,7 +358,7 @@ public class Photo : DbItem, IComparable {
 public class PhotoStore : DbStore {
 
 	TagStore tag_store;
-	static ThumbnailFactory thumbnail_factory = new ThumbnailFactory (ThumbnailSize.Large);
+	public static ThumbnailFactory ThumbnailFactory = new ThumbnailFactory (ThumbnailSize.Large);
 
 
 	// FIXME this is a hack.  Since we don't have Gnome.ThumbnailFactory.SaveThumbnail() in
@@ -400,14 +411,11 @@ public class PhotoStore : DbStore {
 			thumbnail = GenerateFromExif (path, uri);
 
 		if (thumbnail == null)
-			thumbnail = thumbnail_factory.GenerateThumbnail (uri, "image/jpeg");
+			thumbnail = PhotoStore.ThumbnailFactory.GenerateThumbnail (uri, "image/jpeg");
 		
 		// FIXME if this is null then the file doesn't exist.
 		if (thumbnail != null) {
-			// FIXME there is no SaveThumbnail() in the C# bindings for ThumbnailFactory.
-			// This should really be done through SaveThumbnail, which would make sure we dont' do
-			// it unnecessarily.
-			thumbnail.Savev (Thumbnail.PathForUri (uri, ThumbnailSize.Large), "png", null, null);
+			PhotoStore.ThumbnailFactory.SaveThumbnail (thumbnail, uri, File.GetLastWriteTime (path));
 		}
 		
 		return thumbnail;
@@ -491,12 +499,28 @@ public class PhotoStore : DbStore {
 		command.Dispose ();
 	}
 
+	public Photo Create (string path, out Pixbuf thumbnail)
+	{
+		DateTime time;
+		try {
+			using (ExifData ed = new ExifData (path)) {
+				string strtime = ed.LookupString (ExifTag.DateTimeOriginal);
+				time = ExifData.DateTimeFromString (strtime); 
+				time = time.ToUniversalTime ();
+			}			
+		} catch {
+			time = File.GetCreationTimeUtc  (path);
+		} 
+
+		return Create (time, path, out thumbnail);
+	}
 
 	public Photo Create (DateTime time_in_utc, string path, out Pixbuf thumbnail)
 	{
-		if (! path.EndsWith (".jpg") && ! path.EndsWith (".JPG"))
+		if (! path.ToLower().EndsWith (".jpg") && ! path.ToLower().EndsWith (".jpeg"))
 			throw new Exception ("Only jpeg files supported");
 
+
 		uint unix_time = DbUtils.UnixTimeFromDateTime (time_in_utc);
 
 		SqliteCommand command = new SqliteCommand ();
@@ -520,6 +544,164 @@ public class PhotoStore : DbStore {
 		return photo;
 	}
 
+	private void GetVersions (Photo photo)
+	{
+
+		SqliteCommand command = new SqliteCommand ();
+		command.Connection = Connection;
+		command.CommandText = String.Format ("SELECT version_id, name FROM photo_versions WHERE photo_id = {0}", photo.Id);
+		SqliteDataReader reader = command.ExecuteReader ();
+
+		while (reader.Read ()) {
+			uint version_id = Convert.ToUInt32 (reader [0]);
+			string name = reader[1].ToString ();
+
+			photo.AddVersionUnsafely (version_id, name);
+		}
+
+		command.Dispose ();
+	}
+
+	private void GetTags (Photo photo)
+	{
+		SqliteCommand command = new SqliteCommand ();
+		command.Connection = Connection;
+		command.CommandText = String.Format ("SELECT tag_id FROM photo_tags WHERE photo_id = {0}", photo.Id);
+		SqliteDataReader reader = command.ExecuteReader ();
+
+		while (reader.Read ()) {
+			uint tag_id = Convert.ToUInt32 (reader [0]);
+			Tag tag = tag_store.Get (tag_id) as Tag;
+			photo.AddTagUnsafely (tag);
+		}
+
+		command.Dispose ();
+	}		
+	
+	private void GetAllVersions  () {
+		SqliteCommand command = new SqliteCommand ();
+		command.Connection = Connection;
+		command.CommandText = String.Format ("SELECT photo_id, version_id, name " +
+						     "FROM photo_versions ");
+		
+		SqliteDataReader reader = command.ExecuteReader ();
+
+		while (reader.Read ()) {
+			uint id = Convert.ToUInt32 (reader [0]);
+			Photo photo = LookupInCache (id) as Photo;
+				
+			if (photo == null) {
+				//Console.WriteLine ("Photo {0} not found", id);
+				continue;
+			}
+				
+			if (photo.Loaded) {
+				//Console.WriteLine ("Photo {0} already Loaded", photo);
+				continue;
+			}
+
+			if (reader [1] != null) {
+				uint version_id = Convert.ToUInt32 (reader [1]);
+				string name = reader[2].ToString ();
+				
+				photo.AddVersionUnsafely (version_id, name);
+			}
+		}
+	}
+
+	private void GetAllTags () {
+			SqliteCommand command = new SqliteCommand ();
+		command.Connection = Connection;
+		command.CommandText = String.Format ("SELECT photo_id, tag_id " +
+						     "FROM photo_tags ");
+		
+		SqliteDataReader reader = command.ExecuteReader ();
+
+		while (reader.Read ()) {
+			uint id = Convert.ToUInt32 (reader [0]);
+			Photo photo = LookupInCache (id) as Photo;
+				
+			if (photo == null) {
+				//Console.WriteLine ("Photo {0} not found", id);
+				continue;
+			}
+				
+			if (photo.Loaded) {
+				//Console.WriteLine ("Photo {0} already Loaded", photo.Id);
+				continue;
+			}
+
+		        if (reader [1] != null) {
+				uint tag_id = Convert.ToUInt32 (reader [1]);
+				Tag tag = tag_store.Get (tag_id) as Tag;
+				photo.AddTagUnsafely (tag);
+			}
+		}
+	}
+
+	private void GetAllData () {
+		SqliteCommand command = new SqliteCommand ();
+		command.Connection = Connection;
+		command.CommandText = String.Format ("SELECT photo_tags.photo_id, tag_id, version_id, name " +
+						     "FROM photo_tags, photo_versions " +
+						     "WHERE photo_tags.photo_id = photo_versions.photo_id");
+		
+		SqliteDataReader reader = command.ExecuteReader ();
+
+		while (reader.Read ()) {
+			uint id = Convert.ToUInt32 (reader [0]);
+			Photo photo = LookupInCache (id) as Photo;
+				
+			if (photo == null) {
+				//Console.WriteLine ("Photo {0} not found", id);
+				continue;
+			}
+				
+			if (photo.Loaded) {
+				//Console.WriteLine ("Photo {0} already Loaded", photo);
+				continue;
+			}
+
+		        if (reader [1] != null) {
+				uint tag_id = Convert.ToUInt32 (reader [1]);
+				Tag tag = tag_store.Get (tag_id) as Tag;
+				photo.AddTagUnsafely (tag);
+			}
+			if (reader [2] != null) {
+				uint version_id = Convert.ToUInt32 (reader [2]);
+				string name = reader[3].ToString ();
+				
+				photo.AddVersionUnsafely (version_id, name);
+			}
+		}
+	}
+
+	private void GetData (Photo photo)
+	{
+		SqliteCommand command = new SqliteCommand ();
+		command.Connection = Connection;
+		command.CommandText = String.Format ("SELECT tag_id, version_id, name " +
+						     "FROM photo_tags, photo_versions " +
+						     "WHERE photo_tags.photo_id = photo_versions.photo_id " +
+						     "AND photo_tags.photo_id = {0}", photo.Id);
+
+		SqliteDataReader reader = command.ExecuteReader ();
+
+		while (reader.Read ()) {
+		        if (reader [0] != null) {
+				uint tag_id = Convert.ToUInt32 (reader [0]);
+				Tag tag = tag_store.Get (tag_id) as Tag;
+				photo.AddTagUnsafely (tag);
+			}
+			if (reader [1] != null) {
+				uint version_id = Convert.ToUInt32 (reader [1]);
+				string name = reader[2].ToString ();
+				
+				photo.AddVersionUnsafely (version_id, name);
+			}
+		}
+	}
+
 	public override DbItem Get (uint id)
 	{
 		Photo photo = LookupInCache (id) as Photo;
@@ -553,33 +735,57 @@ public class PhotoStore : DbStore {
 
 		if (photo == null)
 			return null;
+		
+		GetTags (photo);
+		GetVersions (photo);
 
-		command = new SqliteCommand ();
+		return photo;
+	}
+
+	public Photo GetByPath (string path)
+	{
+		//FIXME - No cacheing here - probably not a problem since
+		//        this is only used for DND
+
+		Photo photo = null;
+		SqliteCommand command = new SqliteCommand ();
 		command.Connection = Connection;
-		command.CommandText = String.Format ("SELECT tag_id FROM photo_tags WHERE photo_id = {0}", id);
-		reader = command.ExecuteReader ();
 
-		while (reader.Read ()) {
-			uint tag_id = Convert.ToUInt32 (reader [0]);
-			Tag tag = tag_store.Get (tag_id) as Tag;
-			photo.AddTagUnsafely (tag);
+		string directory_path = System.IO.Path.GetDirectoryName (path);
+		string filename = System.IO.Path.GetFileName (path);
+
+		directory_path = directory_path.Trim();
+		filename = filename.Trim();
+		command.CommandText = String.Format ("SELECT id,                                   " +
+				                     "       time,                                 " +
+						     "       description,                          " +
+						     "       default_version_id                    " +
+						     "  FROM photos                                " +
+						     " WHERE directory_path = \"{0}\"              " +
+						     "   AND name = \"{1}\"                        ",
+						     directory_path,
+						     filename);
+
+		SqliteDataReader reader = command.ExecuteReader ();
+
+		if (reader.Read ()) {
+			photo = new Photo (Convert.ToUInt32 (reader [0]),
+					   Convert.ToUInt32 (reader [1]),
+					   directory_path,
+					   filename);
+
+			photo.Description = reader[2].ToString ();
+			photo.DefaultVersionId = Convert.ToUInt32 (reader[3]);
+			AddToCache (photo);
 		}
 
 		command.Dispose ();
 
-		command = new SqliteCommand ();
-		command.Connection = Connection;
-		command.CommandText = String.Format ("SELECT version_id, name FROM photo_versions WHERE photo_id = {0}", id);
-		reader = command.ExecuteReader ();
-
-		while (reader.Read ()) {
-			uint version_id = Convert.ToUInt32 (reader [0]);
-			string name = reader[1].ToString ();
-
-			photo.AddVersionUnsafely (version_id, name);
-		}
-
-		command.Dispose ();
+		if (photo == null)
+			return null;
+		
+		GetTags (photo);
+		GetVersions (photo);
 
 		return photo;
 	}
@@ -598,22 +804,25 @@ public class PhotoStore : DbStore {
 		
 	}
 
-#if false
 	public void Remove (Photo []items)
 	{
 		StringBuilder query_builder = new StringBuilder ();
-
+		StringBuilder tv_query_builder = new StringBuilder ();
 		for (int i = 0; i < items.Length; i++) {
-			if (i > 0)
+			if (i > 0) {
 				query_builder.Append (" OR ");
-			query_builder.Append (items[i].Id);
+				tv_query_builder.Append (" OR ");
+			}
+
+			query_builder.Append (String.Format ("id = {0}", items[i].Id));
+			tv_query_builder.Append (String.Format ("photo_id = {0}", items[i].Id));
 			RemoveFromCache (items[i]);
 		}
 
 		SqliteCommand command = new SqliteCommand ();
 		command.Connection = Connection;
 
-		command.CommandText = String.Format ("DELETE FROM photos WHERE id = {0}", query_builder.ToString ());
+		command.CommandText = String.Format ("DELETE FROM photos WHERE {0}", query_builder.ToString ());
 		command.ExecuteNonQuery ();
 
 		command.Dispose ();
@@ -621,7 +830,7 @@ public class PhotoStore : DbStore {
 		command = new SqliteCommand ();
 		command.Connection = Connection;
 
-		command.CommandText = String.Format ("DELETE FROM photo_tags WHERE photo_id = {0}", query_builder.ToString ());
+		command.CommandText = String.Format ("DELETE FROM photo_tags WHERE {0}", tv_query_builder.ToString ());
 		command.ExecuteNonQuery ();
 
 		command.Dispose ();
@@ -629,12 +838,11 @@ public class PhotoStore : DbStore {
 		command = new SqliteCommand ();
 		command.Connection = Connection;
 
-		command.CommandText = String.Format ("DELETE FROM photo_versions WHERE photo_id = {0}", query_builder.ToString ());
+		command.CommandText = String.Format ("DELETE FROM photo_versions WHERE {0}", tv_query_builder.ToString ());
 		command.ExecuteNonQuery ();
 
 		command.Dispose ();
 	}
-#endif
 
 	public override void Remove (DbItem item)
 	{
@@ -723,78 +931,168 @@ public class PhotoStore : DbStore {
 			command.Dispose ();
 		}
 	}
+	
+	public class DateRange 
+	{
+		private DateTime start;		
+		public DateTime Start {
+			get {
+				return start;
+			}
+		}
 
+		private DateTime end;
+		public DateTime End {
+			get {
+				return end;
+			}
+		}
+
+		public DateRange (DateTime start, DateTime end)
+		{
+			this.start = start;
+			this.end = end;
+		}
+	}
 
 	// Queries.
+	public Photo [] Query (Tag [] tags, DateTime start, DateTime end)
+	{
+		return Query (tags, new DateRange (start, end));
+	}
+	
+	public Photo [] Query (Tag [] tags) {
+		return Query (tags, null);
+	}
 
-	public Photo [] Query (Tag [] tags)
+	public Photo [] Query (Tag [] tags, DateRange range)
 	{
 		string query;
 
-		if (tags == null || tags.Length == 0) {
-			query = "SELECT id FROM photos";
-		} else {
-			// The SQL query that we want to construct is:
-			//
-			// SELECT photos.id FROM photos, photo_tags
-			//                  WHERE photos.id = photo_tags.photo_id
-			// 		          AND (photo_tags.tag_id = tag1
-			//			       OR photo_tags.tag_id = tag2
-			//                             OR photo_tags.tag_id = tag3 ...)
-			//                  GROUP BY photos.id
-
-			StringBuilder query_builder = new StringBuilder ();
-			query_builder.Append ("SELECT photos.id FROM photos, photo_tags               " +
-					      "                 WHERE photos.id = photo_tags.photo_id ");
-
-			if (tags != null) {
+		bool hide = true;
+		if (tags != null) {
+			foreach (Tag t in tags) {
+				if (t.Id == tag_store.Hidden.Id) 
+					hide = false;
+			}
+		}
+		
+		// The SQL query that we want to construct is:
+		//
+		// SELECT photos.id
+		//        photos.time
+		//        photos.directory_path,
+		//        photos.name,
+		//        photos.description,
+		//        photos.default_version_id
+		//                  FROM photos, photo_tags
+		//                  WHERE photos.id = photo_tags.photo_id
+		// 		          AND (photo_tags.tag_id = tag1
+		//			       OR photo_tags.tag_id = tag2
+		//                             OR photo_tags.tag_id = tag3 ...)
+		//                  GROUP BY photos.id
+		
+		StringBuilder query_builder = new StringBuilder ();
+		query_builder.Append ("SELECT photos.id,                          " +
+				      "       photos.time,                        " +
+				      "       photos.directory_path,              " +
+				      "       photos.name,                        " +
+				      "       photos.description,                 " +
+				      "       photos.default_version_id           " +
+				      "     FROM photos                      ");
+		
+		if (range != null) {
+			query_builder.Append (String.Format ("WHERE photos.time >= {0} AND photos.time < {1} ",
+							     DbUtils.UnixTimeFromDateTime (range.Start), 
+							     DbUtils.UnixTimeFromDateTime (range.End)));
+		}
+		
+		if (hide) {
+			query_builder.Append (String.Format ("{0} photos.id NOT IN (SELECT photo_id FROM photo_tags WHERE tag_id = {1})", 
+							     range != null ? " AND " : " WHERE ", tag_store.Hidden.Id));
+		}
+		
+		if (tags != null && tags.Length > 0) {
 				bool first = true;
 				foreach (Tag t in tags) {
-					if (first)
-						query_builder.Append (" AND (");
-					else
-						query_builder.Append (" OR ");
-
-					query_builder.Append (String.Format ("photo_tags.tag_id = {0}", t.Id));
-
+					if (t.Id == tag_store.Hidden.Id)
+						continue;
+					
+					if (first) {
+						query_builder.Append (String.Format ("{0} photos.id IN (SELECT photo_id FROM photo_tags WHERE tag_id IN (",
+										     hide || range != null ? " AND " : " WHERE "));
+					}
+					
+					query_builder.Append (String.Format ("{0}{1} ", first ? "" : ", ", t.Id));
+					
 					first = false;
 				}
-
-				if (tags.Length > 0)
-					query_builder.Append (")");
-			}
-
-			query_builder.Append (" GROUP BY photos.id");
-			query = query_builder.ToString ();
+				
+				if (!first)
+					query_builder.Append (")) ");
 		}
+		
+		query_builder.Append ("ORDER BY photos.time");
+		query = query_builder.ToString ();
+		
+		Console.WriteLine ("Query Start {0}", System.DateTime.Now.ToLongTimeString ());
 
 		SqliteCommand command = new SqliteCommand ();
 		command.Connection = Connection;
 		command.CommandText = query;
 		SqliteDataReader reader = command.ExecuteReader ();
-
-		// FIXME: I am doing it in two passes here since the Get() method can potentially
-		// invoke more Sqlite queries, and I don't know if we are supposed to do that while
-		// we are reading the results from a past query.
 		
+		Console.WriteLine ("Query Mid {0}", System.DateTime.Now);
+
+		ArrayList version_list = new ArrayList ();
 		ArrayList id_list = new ArrayList ();
-		while (reader.Read ())
-			id_list.Add (Convert.ToUInt32 (reader [0]));
+		while (reader.Read ()) {
+			uint id = Convert.ToUInt32 (reader [0]);
+			Photo photo = LookupInCache (id) as Photo;
+
+			if (photo == null) {
+				photo = new Photo (id,
+						   Convert.ToUInt32 (reader [1]),
+						   reader [2].ToString (),
+						   reader [3].ToString ());
+				
+				photo.Description = reader[4].ToString ();
+				photo.DefaultVersionId = Convert.ToUInt32 (reader[5]);		 
+				
+				version_list.Add (photo);
+			}
+
+			id_list.Add (photo);
+		}
+
+		Console.WriteLine ("Query End {0}", System.DateTime.Now.ToLongTimeString ());
+
+		bool need_load = false;
+		Console.WriteLine ("Start {0}", System.DateTime.Now);
+		foreach (Photo photo in version_list) {
+			AddToCache (photo);
+			need_load |= !photo.Loaded;
+		}
+		
+		if (need_load) {
+			GetAllTags ();
+			GetAllVersions ();
+			foreach (Photo photo in version_list) {
+				photo.Loaded = true;
+			}
+		} else {
+			//Console.WriteLine ("Skipped Loading Data");
+		}
+
+
+		Console.WriteLine ("End {0}", System.DateTime.Now);
 
 		command.Dispose ();
 
-		Photo [] photo_list = new Photo [id_list.Count];
-		int i = 0;
-		foreach (uint id in id_list)
-			photo_list [i ++] = Get (id) as Photo;
-
-		//Array.Sort (photo_list);
-		return photo_list;
+		return id_list.ToArray (typeof (Photo)) as Photo [];
 	}
 
-
 #if TEST_PHOTO_STORE
-
 	static void Dump (Photo photo)
 	{
 		Console.WriteLine ("\t[{0}] {1}", photo.Id, photo.Path);
