@@ -14,6 +14,7 @@ public class SlideView : Gtk.Image {
 	Pixbuf [] tweens = new Pixbuf [10];	
 	int current_tween;
 	uint tween_idle;
+	uint resize_idle;
 
 	int current_idx = 0;	
 	int next_idx = 0;
@@ -21,6 +22,10 @@ public class SlideView : Gtk.Image {
 	uint flip_timer = 0;
 	uint transition_timer = 0;
 
+	bool black = false;
+	uint flip_interval = 2000;
+	uint transition_interval = 75;
+
 	public bool Running {
 		get {
 			return flip_timer != 0 || transition_timer != 0;
@@ -30,7 +35,11 @@ public class SlideView : Gtk.Image {
 	public void Play () 
 	{
 		StopTweenIdle ();
-		this.FromPixbuf = GetScaled (photos[current_idx].DefaultVersionPath);
+		if (current_idx >= 0) {
+			Pixbuf frame = GetScaled (photos[current_idx]);
+			this.FromPixbuf = frame;
+			frame.Dispose ();
+		} 
 
 		if (PreloadNextImage (current_idx + 1))
 			StartFlipTimer ();
@@ -69,39 +78,53 @@ public class SlideView : Gtk.Image {
 			this.FromPixbuf = next;
 
 		current_idx = next_idx;
+
+		black = false;
+		transition_interval = 75;
+		flip_interval = 2000;
 	}
 
 	private bool PreloadNextImage (int idx)
 	{
 		Pixbuf orig;
 	
-		if (idx < photos.Length && idx >= 0) {
-			//Console.WriteLine ("next_idx = " + next_idx + " idx = " + idx);
-			next = GetScaled (photos [idx].DefaultVersionPath);
-	
-			next_idx = idx;
-			StartTweenIdle ();
-			return true;
-		} else {
-			//Console.WriteLine ("What happens now?");
-			next = null;
-			next_idx = 0;
-			return false;
+		try {
+			if (idx < photos.Length && idx >= 0) {
+				//Console.WriteLine ("next_idx = " + next_idx + " idx = " + idx);
+				next = GetScaled (photos [idx]);
+				
+				next_idx = idx;
+				StartTweenIdle ();
+				
+
+				return true;
+			} else {
+				//Console.WriteLine ("What happens now?");
+				next = GetScaled (photos [0]);
+				next_idx = 0;
+				StartTweenIdle ();
+				
+				return false;
+			}
+		} catch (GLib.GException e) {
+			Console.WriteLine (e);
+			idx = (idx + 1) % photos.Length;
+			return PreloadNextImage (idx);
 		}
 	}
-
-	private Pixbuf Blend (Pixbuf current, Pixbuf prev, Pixbuf next, double percent)
+	
+	private Pixbuf CrossFade (Pixbuf current, Pixbuf prev, Pixbuf next, double percent)
 	{ 
 		int width = Allocation.Width;
 		int height = Allocation.Height;
 
 		prev.CopyArea (0, 0, width, height, current, 0, 0);
 		next.Composite (current, 0,0, width, height, 0, 0, 1, 1,
-				Gdk.InterpType.Bilinear, (int)(255 * percent + 0.5));
+				Gdk.InterpType.Nearest, (int)(255 * percent + 0.5));
 		return current;
 	}
 
-	private Pixbuf FadeBlack (Pixbuf current, Pixbuf prev, Pixbuf next, double percent)
+	private Pixbuf BlackFade (Pixbuf current, Pixbuf prev, Pixbuf next, double percent)
 	{ 
 		int width = Allocation.Width;
 		int height = Allocation.Height;
@@ -110,32 +133,32 @@ public class SlideView : Gtk.Image {
 
 		if (percent < 0.5)
 			prev.Composite (current, 0,0, width, height, 0, 0, 1, 1,
-					Gdk.InterpType.Bilinear, (int)(255 * (1 - percent * 2) + 0.5));
+					Gdk.InterpType.Nearest, (int)(255 * (1 - percent * 2) + 0.5));
 		else
 			next.Composite (current, 0,0, width, height, 0, 0, 1, 1,
-					Gdk.InterpType.Bilinear, (int)(255 * (percent * 2 - 1) + 0.5));
+					Gdk.InterpType.Nearest, (int)(255 * (percent * 2 - 1) + 0.5));
 		return current;
 	}
 
-	private Pixbuf GetScaled (string path)
+	private Pixbuf Blend (Pixbuf current, Pixbuf prev, Pixbuf next, double percent)
+	{
+		if (black) {
+			return BlackFade (current, prev, next, percent);
+		} else {
+			return CrossFade (current, prev, next, percent);
+		}
+	}
+
+	private Pixbuf GetScaled (Pixbuf orig)
 	{
 		int width = Allocation.Width;
 		int height = Allocation.Height;
-
-		Pixbuf orig;
 		Pixbuf scaled = new Pixbuf (Colorspace.Rgb, false, 8, width, height);
 		scaled.Fill (0);
 		
 		if (width < 10 || height < 10)
 			return scaled;	
 
-		try {
-		orig = new Pixbuf (path);
-		} catch {
-			Console.WriteLine ("Error loading file " + path);
-			orig = null;
-		}
-
 		if (orig == null) {
 			return scaled;
 		}
@@ -158,6 +181,16 @@ public class SlideView : Gtk.Image {
 		return scaled;
 	}
 
+	private Pixbuf GetScaled (Photo photo)
+	{
+		Pixbuf orig = FSpot.PhotoLoader.LoadAtMaxSize (photo, Allocation.Width, Allocation.Height);
+		Pixbuf result = GetScaled (orig);
+		if (orig != result)
+			orig.Dispose ();
+
+		return result;
+	}
+
 	private bool HandleFlipTimer ()
 	{	
 		StopTweenIdle ();
@@ -188,8 +221,7 @@ public class SlideView : Gtk.Image {
 
 	private bool HandleTweenIdle ()
 	{
-		Pixbuf prev = this.Pixbuf;
-	
+		using (Pixbuf prev = this.Pixbuf) {	
 		if (current_tween < tweens.Length && tweens[current_tween] == null) {
 			tweens[current_tween] = new Pixbuf (Colorspace.Rgb, false, 8, Allocation.Width, Allocation.Height);
 		}
@@ -232,6 +264,7 @@ public class SlideView : Gtk.Image {
 
 		current_tween++;
 		return true;
+		}
 	}	
 
 	private void StartTweenIdle () 
@@ -254,7 +287,8 @@ public class SlideView : Gtk.Image {
 	private void StartTransitionTimer ()
 	{
 		if (transition_timer == 0)
-			transition_timer = GLib.Timeout.Add (75, new TimeoutHandler (HandleTransitionTimer));
+			transition_timer = GLib.Timeout.Add (transition_interval, 
+							     new TimeoutHandler (HandleTransitionTimer));
 	}
 
 	private void StopTranstionTimer ()
@@ -268,7 +302,8 @@ public class SlideView : Gtk.Image {
 	private void StartFlipTimer ()
 	{
 		if (flip_timer == 0)
-			flip_timer = GLib.Timeout.Add (2000, new TimeoutHandler (HandleFlipTimer));
+			flip_timer = GLib.Timeout.Add (flip_interval, 
+						       new TimeoutHandler (HandleFlipTimer));
 	}
 	
 	private void StopFlipTimer ()
@@ -278,39 +313,67 @@ public class SlideView : Gtk.Image {
 		}
 		flip_timer = 0;
 	}
-	
+
+
 	private void HandleSizeAllocate (object sender, SizeAllocatedArgs args)
 	{	
 		if (Pixbuf == null)
 			return;
 	
-		/*
-		 * The size has changed so we need to reload the images.
-		 */
-		if (Pixbuf.Width != Allocation.Width || Pixbuf.Height != Allocation.Height) {
-			bool playing = (flip_timer != 0 || transition_timer != 0);
-
-			this.FromPixbuf = GetScaled (photos[current_idx].DefaultVersionPath);
-			Stop ();
-
-			/* clear the tween images */
-			for (int i = 0; i < tweens.Length; i++)
-				tweens[i] = null;
-
+		//
+		// The size has changed so we need to reload the images.
+		//
+		Pixbuf current = this.Pixbuf;
+		if (current.Width != Allocation.Width || current.Height != Allocation.Height) {
+				bool playing = (flip_timer != 0 || transition_timer != 0);
+				
+				if (current_idx < 0) {
+					this.FromPixbuf = GetScaled (this.Pixbuf);
+				} else {
+					using (Pixbuf frame =  GetScaled (photos[current_idx])) {
+						this.FromPixbuf =  frame;
+						current.Dispose ();
+					}
+				}
+				
+				Stop ();
+		
+			ClearTweens ();
+			
 			if (playing && current_idx != next_idx)
 				Play ();
+
+
+		}
+	}
+
+	private void ClearTweens () {
+		for (int i = 0; i < tweens.Length; i++) {
+			if (tweens[i] != null) 
+				tweens[i].Dispose ();
+			tweens[i] = null;
 		}
 	}
 
 	private void HandleDestroyed (object sender, EventArgs args)
 	{
+		ClearTweens ();
 		Stop ();
 	}
 
-	public SlideView (Photo [] photos) : base ()
+	public SlideView (Pixbuf background, Photo [] photos) : base ()
 	{
 		this.photos = photos;
 
+		if (background != null) {
+			this.FromPixbuf = background;
+			background.Dispose ();
+
+			current_idx = -1;
+			black = true;
+			flip_interval = 1500;
+		}
+
 		SizeAllocated += new SizeAllocatedHandler (HandleSizeAllocate);
 		Destroyed += new EventHandler (HandleDestroyed);
 	}
